/* Generation of all permutations of a set of n elements in lexicographic order. */

#include <stdio.h>
// Additional library for time counting
#include <time.h>

#include "perm.h"
// Auxiliary functions:
#include "../../fcts/fcts.h"

#define MAX 16

int main(int argc,char *argv[]) {
 int c, n;
 int p[MAX];
  
 // Variables for time computation:
 clock_t lasttime, now, diff;
 
 if (argc != 2) {
  printf("Invalid number of arguments.\n");
  printf("Usage: perm [n]\n");
  printf("Generation of all permutations of a set of n elements.\n");
  return(-1);
 }

 n = atoi(argv[argc-1]);
 if (n < 1) {
  printf("Invalid first argument: The minimal size should be 1 or more.\n");
  return(-1);
 }
 if (n > MAX) {
  printf("Invalid first argument: The minimal size should be structurally %d or less.\n",MAX);
  return(-1);
 }

 printf("Permutations of a set of %d elements:\n",n);
 printf("\n");  
 
 first_perm(p,n);   
 displayl(p,n);
 printf("\n");

 c = 1; // we count the words generated
 lasttime = clock(); 

 while(next_perm(p,n) == 1) {
  displayl(p,n);
  printf("\n");
  c++;
 }
  
 printf("Number of permutations of a set of %d elements generated: %d\n", n, c);
 if (fact(n) == c) {
  printf("Equal to %d!.\n",n);
 } else {
  printf("Not equal to %d!.\n",n); 
 }
 now = clock();
 diff = (now - lasttime) / (CLOCKS_PER_SEC / 1000);
 printf("Time (ms): %ld\n", (long) diff);
 printf("\n");

 return 0;
}
