/* Generation of all subsets of a set of n elements from delta sets, in
 * the lexicographic increasing order (on the delta sets) induced by the
 * precedence 0 < 1. Adaptation of the algorithm page 203 of the fxtbook.
 */

#include "subset.h"

int first_subset(int a[], int n) {
 int k;
 /*@ loop invariant 0 <= k <= n;
   @ loop invariant is_dset(a,k);
   @ loop assigns k, a[0..n-1];
   @ loop variant n-k; */
 for ( k = 0; k < n; k++) {
  a[k] = 0;
 }
 return 1;
}


int next_subset(int a[], int n) {
 int i,k;

 /*@ loop invariant -1 <= k <= n-1;
   @ loop assigns k;
   @ loop variant k; */
 for (k = n-1; k >= 0; k--) { if (a[k] == 0) { break; } }
 if (k == -1) { return 0; }
 a[k] = 1;
 /*@ loop invariant k+1 <= i <= n;
   @ loop invariant is_dset(a,i);
   @ loop assigns i,a[k+1..n-1];
   @ loop variant n-i; */
 for (i = k+1; i < n; i++) { a[i] = 0; }
 /*@ assert lt_lex_at{Pre,Here}(a,k); */
 return 1;
}
