/* Generation of all functions from [0..n-1] to [0..k-1], in the increasing
 * lexicographic order induced by the precedence order < on integers.
 * Each function is tabulated in an array of size n containing values between 0
 * and k-1. */

#include "fct.h"

int first_fct(int a[], int n, int k) {
 int i;
 /*@ loop invariant 0 <= i <= n;
   @ loop invariant is_fct(a,i,k);
   @ loop assigns i, a[0..n-1];
   @ loop variant n-i; */
 for (i = 0; i < n; i++) {
  a[i] = 0;
 }
 return 1;
}

/* Returns 1 if the array is modified, or 0 if the array
   is unchanged. */
int next_fct(int a[], int n, int k) {
 int i,j;

 // Find rightmost digit that can be incremented:
 /*@ loop invariant -1 <= j <= n-1;
   @ loop assigns j;
   @ loop variant j; */
 for (j = n-1; j >= 0; j--) {
  if (a[j] < k-1) { break; }
 }
 // If there is no such index, then a = [k-1,...,k-1] is the last fct.
 if (j == -1) { return 0; }
 a[j] = a[j] + 1;
 // Fill the suffix starting at a[j+1] with zeros:
 /*@ loop invariant j+1 <= i <= n;
   @ loop invariant is_fct(a,i,k);
   @ loop assigns i,a[j+1..n-1];
   @ loop variant n-i; */
 for (i = j+1; i < n; i++) { a[i] = 0; }
 /*@ assert lt_lex_at{Pre,Here}(a,j); */
 return 1;
}
