/* Efficient generation algorithm for sorted arrays from [0..n-1] to [0..k-1]. */

#include "sorted.h"

int first_sorted(int a[], int n, int k) {
 int i;
 /*@ loop invariant 0 <= i <= n;
   @ loop invariant is_zero(a,i);
   @ loop assigns i, a[0..n-1];
   @ loop variant n-i; */
 for (i = 0; i < n; i++) {
  a[i] = 0;
 }
 return 1;
}

int next_sorted(int a[], int n, int k) {
 int i,j;

 // Find rightmost digit that can be incremented:
 /*@ loop invariant -1 <= j <= n-1;
   @ loop assigns j;
   @ loop variant j; */
 for (j = n-1; j >= 0; j--) {
  if (a[j] < k-1) { break; }
 }
 // If there is no such index, then a = [k-1,...,k-1] is the last sorted array.
 if (j == -1) { return 0; }
 a[j] = a[j] + 1;

 // Fill the suffix starting at a[j+1] with a[j]:
 /*@ loop invariant j+1 <= i <= n;
   @ loop invariant is_fct(a,i,k);
   @ loop invariant is_eq_aj(a,j+1,i,a[j]);
   @ loop invariant is_le(a,j+1,i);
   @ loop assigns i,a[j+1..n-1];
   @ loop variant n-i; */
 for (i = j+1; i < n; i++) { a[i] = a[j]; }
 return 1;
}

